//artikulation player
//kevin schlei
//updated: 10.8.2018

import processing.opengl.*;

ArrayList buffers;

float scrollRate;

//!
float tilterCounter;

PImage[] artikulationCropped;
float[] artikulationImageScalers;
int artikulationScoreCounter;
PImage textureImage;

PFont titleFont;
boolean isPlaying;
boolean playScore;
boolean playGenerative;

//globals:
color bgColor;
boolean showRects, showRandBufferBackground;
int minimumBufferWidth;
color[] allColors, dotColors, combColors;
int[] dotColorRandom, combColorRandom;
int dotColorCycler, combColorCycler, lastDotColorIndex, lastCombColorIndex;
int buffersGenerated, maxGeneratedBuffers;
float tiltTint;

Shape getMainShape() {
  Shape s;
  float r = random(1);
  if (r >= 0.0 && r < 0.15) {
    s = new DotCurvedTail();
    s.setSizes(40, 35, 90, 60);
  } else if (r >= 0.15 && r < 0.3) {
    s = new DotStraightTail();
    s.setSizes(40, 20, 190, 32);
  } else if (r >= 0.3 && r < 0.4) {
    s = new BlocksTailGroup();
    s.setSizes(200, 100, 500, int(height * 0.75));
  } else if (r >= 0.4 && r < 0.5) {
    s = new Comb();
    s.setSizes(20, 40, 40, 50);
  } else {
    s = new DotsConnected();
    s.setSizes(35, 60, 110, 110);
  }
  return s;
}

Shape getLowerShape() {
  Shape s;
  s = new Wedges();
  s.setSizes(18, 40, 55, 70);
  return s;
}

Shape getUpperShape() {
  Shape s;
  float r = random(1);
  if (r >= 0.0 && r <= 0.4) {
    s = new Blocks();
    s.setSizes(20, 35, 60, 90);
  } else if (r >= 0.4 && r <= 0.5) {
    s = new Block();
    s.setSizes(19, 19, 21, 21);
  } else if (r >= 0.5 && r <= 0.6) {
    s = new Dot();
    s.setSizes(15, 15, 35, 35);
  } else {
    s = new Comb();
    s.setSizes(15, 25, 45, 70);
  }
  return s;
}

color getDotColor() {
  if (dotColorCycler == 0) {
    for (int i=0; i<dotColors.length; i++) {
      if (true) lastDotColorIndex = int(random(dotColors.length));
      dotColorRandom[i] = lastDotColorIndex;
    }
  }
  color c = dotColors[dotColorRandom[dotColorCycler]];
  //  dotColorCycler = (dotColorCycler + 1) % dotColorRandom.length;

  return c;
}

color getCombColor() {
  if (combColorCycler == 0) {
    for (int i=0; i<combColors.length; i++) {
      if (true) lastCombColorIndex = int(random(combColors.length));
      combColorRandom[i] = lastCombColorIndex;
    }
  }
  color c = combColors[combColorRandom[combColorCycler]];
  //  combColorCycler = (combColorCycler + 1) % combColorRandom.length;

  return c;
}

void setup() {
  size(1024, 320, P2D);
  background(255);
  noCursor();
  frame.setBackground(new java.awt.Color(0, 0, 0));

  ///////////////////
  //DEFAULTS
  ///////////////////

  bgColor = color(255, 255, 255);
  showRects = false;
  showRandBufferBackground = false;
  scrollRate = 1.4; //in pixels
  minimumBufferWidth = 100;
  maxGeneratedBuffers = 50;
  ///////////////////
  ///////////////////

  textureImage = loadImage("texture.png");

  dotColorRandom = new int[20];
  combColorRandom = new int[20];
  lastDotColorIndex = 0;
  lastCombColorIndex = 0;
  buffersGenerated = 0;
  tiltTint = 128.0;

  allColors = new color[10];
  allColors[0] = color(#794a39); //dark brown
  allColors[1] = color(#db0000); //red
  allColors[2] = color(#d45f00); //orange
  allColors[3] = color(#b99170); //light brown
  allColors[4] = color(#ead442); //yellow
  allColors[5] = color(#94ad53); //light green
  allColors[6] = color(#5b8257); //dark green
  allColors[7] = color(#5c6396); //blue (origin 97,96,148
  allColors[8] = color(#ba6891); //pink
  allColors[9] = color(0, 0, 0); //black

  dotColors = new color[6];
  dotColors[0] = allColors[1];
  dotColors[1] = allColors[2];
  dotColors[2] = allColors[4];
  dotColors[3] = allColors[7];
  dotColors[4] = allColors[8];
  dotColors[5] = allColors[9];

  combColors = new color[5];
  combColors[0] = allColors[0];
  combColors[1] = allColors[3];
  combColors[2] = allColors[5];
  combColors[3] = allColors[6];
  combColors[4] = allColors[9];

  //  frameRate(50);
  buffers = new ArrayList();

  artikulationCropped = new PImage[21];
  artikulationImageScalers = new float[21];
  for (int i=0; i<21; i++) {
    artikulationCropped[i] = loadImage("art-"+i+".png");
  }
  artikulationScoreCounter = 0;

  titleFont = createFont("Menlo", 32);
  textFont(titleFont, 32);

  noLoop(); //keypressed will start playback with options

  isPlaying = false;
  playScore = false;
  playGenerative = false;

  tilterCounter = 0;
}

void draw() {
    background(255);
    println(frameRate);
    
  if (buffersGenerated > maxGeneratedBuffers) {
    //    float tilt = max(1, ((float(mouseY) / 4) / height) * 255.0);
    tiltTint -= 0.03;
    scrollRate += 0.001;
    tiltTint = max(1, tiltTint);
    //    blend(0,0,ADD);
    tint(255, tiltTint);
  }

  //if a buffer area is in range, display it:
  for (int i=0; i<buffers.size(); i++) {
    BufferArea buf = (BufferArea)buffers.get(i);
    if (buf.scrollPosition < width) {
      image(buf.displayImage, buf.scrollPosition, 0, buf.bufferWidth, height);
    }
    buf.scrollPosition -= scrollRate;
  }

  if (buffers.size() > 0) {
    //if it's scrolled out of sight, remove it:
    BufferArea firstBuf = (BufferArea)buffers.get(0);
    if (firstBuf.scrollPosition + firstBuf.bufferWidth < 0) {
      buffers.remove(0);
      //      println("remove buffer new list length is: "+buffers.size());
    }

    //check again that we're not dealing w/ empty array:
    if (buffers.size() > 0) {
      //if we need a new buffer to fill the screen, add it:
      firstBuf = (BufferArea)buffers.get(0);
      float totalWidth = firstBuf.scrollPosition;
      for (int i=0; i<buffers.size(); i++) {
        BufferArea buf = (BufferArea)buffers.get(i);
        totalWidth += buf.bufferWidth;
      }
      if (totalWidth < width + (scrollRate * 3)) {
        BufferArea previousBuffer = (BufferArea)buffers.get(buffers.size()-1);
        if (artikulationScoreCounter < 21 &&
          playScore) {
          artikulationImageScalers[artikulationScoreCounter] = float(height) / float(artikulationCropped[artikulationScoreCounter].height);
          println(artikulationImageScalers[artikulationScoreCounter]);
          buffers.add(new BufferArea(abs(int(artikulationCropped[artikulationScoreCounter].width * artikulationImageScalers[artikulationScoreCounter])), height, previousBuffer.scrollPosition + previousBuffer.bufferWidth));
          BufferArea buf = (BufferArea)buffers.get(buffers.size()-1);
          buf.displayImage = artikulationCropped[artikulationScoreCounter];
          artikulationScoreCounter++;
        } else if (playGenerative) {
          buffers.add(new BufferArea(int(random(min(500, width))) + minimumBufferWidth, height, previousBuffer.scrollPosition + previousBuffer.bufferWidth));
          buffersGenerated++;
        }
      }
    }
  }

  if (isPlaying) {
    //playback line
    if (buffersGenerated <=0) {
      strokeWeight(1);
      stroke(0, 30);
      line(width/2, 0, width/2, height);
    } else if (buffersGenerated > 0 &&
      buffersGenerated <= maxGeneratedBuffers) {
      strokeWeight(1);
      stroke(0, 30);
      line(width/2, 0, width/2, height);
    } else {
      strokeWeight(1);
      stroke(255, tiltTint);
      line(width/2, 0, width/2, height);
    }
  } else {
    //title text
    fill(0);
    textAlign(CENTER);
    textFont(titleFont, 32);
    text("Artikulation", width/2, height/2.5);

    fill(0, 48);
    textFont(titleFont, 10);
    text("Gyorgy Ligeti 1958, Rainer Wehinger 1970, Kevin Schlei 2012", width/2, height - 20);


    fill(0);
    textAlign(CENTER);
    //    text("Artikulation", width/2, height/2.5);

    textFont(titleFont, 10);
    text("Spacebar: play all", width/2, height/2.0 + 60);
    text("S: play score only", width/2, height/2.0 + 75);
    text("G: play generative only", width/2, height/2.0  + 90);

    PImage textureSmall = loadImage("texture-small.png");

    noStroke();
    float cx, ss, off;
    cx = width/2.0;
    ss = 20;
    off = 11;
    for (int i=0; i<allColors.length; i++) {
      //      for (int j=0; j<2; j++) {
      fill(allColors[i]);
      float ox = (cx + (ss+off)*i) - ((ss+off)*allColors.length - off)/2.0;
      float oy = height/2.0;

      rect(ox, oy, ss, ss);

      tint(255, 75);
      image(textureSmall, ox, oy, ss, ss);
      tint(255);
    }
  }

  //  println(buffers.size());
  //  println(frameRate);
}

void keyPressed() {
  //always clear buffer
  buffers.clear();
  artikulationScoreCounter = 0;

  println("keyCode "+keyCode);
  switch(keyCode) {
  case 83: //s
    {
      playScore = true;
      playGenerative = false;

      //add first buffer, make it score:
      artikulationImageScalers[artikulationScoreCounter] = float(height) / float(artikulationCropped[artikulationScoreCounter].height);
      println(artikulationImageScalers[artikulationScoreCounter]);
      buffers.add(new BufferArea(abs(int(artikulationCropped[artikulationScoreCounter].width * artikulationImageScalers[artikulationScoreCounter])), height, width));
      BufferArea buf = (BufferArea)buffers.get(buffers.size()-1);
      buf.displayImage = artikulationCropped[artikulationScoreCounter];
      artikulationScoreCounter++;

      isPlaying = true;
      loop();
    }
    break;
  case 71: //g
    {
      playGenerative = true;
      playScore = false;

      //add first buff generative
      buffers.add(new BufferArea(100, height, width));

      isPlaying = true;
      loop();
    }
    break;
  case 32: //space
    {
      playScore = true;
      playGenerative = true;

      //add first buffer, make it score:
      artikulationImageScalers[artikulationScoreCounter] = float(height) / float(artikulationCropped[artikulationScoreCounter].height);
      println(artikulationImageScalers[artikulationScoreCounter]);
      buffers.add(new BufferArea(abs(int(artikulationCropped[artikulationScoreCounter].width * artikulationImageScalers[artikulationScoreCounter])), height, width));
      BufferArea buf = (BufferArea)buffers.get(buffers.size()-1);
      buf.displayImage = artikulationCropped[artikulationScoreCounter];
      artikulationScoreCounter++;

      isPlaying = true;
      loop();
    }
    break;
  default:
    break;
  }

  isPlaying = true;
  loop();
}
